/**
 * \file gre/lrregion.h
 * \brief GRE Region Layer definitions
 *
 * \if NODOC
 * $Id: lrregion.h_v 1.20 2011/10/20 13:26:14 mju Exp $
 *
 * $Log: lrregion.h_v $
 * Revision 1.20  2011/10/20 13:26:14  mju
 * Remove redundant hdr.
 *
 * Revision 1.19  2006/10/09 17:51:18  mju
 * Rmv v_setDftName.
 *
 * Revision 1.18  2006/10/02 14:14:15  mju
 * Use rvc::objitem.
 *
 * Revision 1.17  2006/09/15 15:30:43  mju
 * Serializer changes.
 *
 * Revision 1.16  2005/11/28 16:38:22  mju
 * Treeview update legend.
 *
 * Revision 1.15  2005/10/26 20:41:13  mju
 * Fix inclusion guard tests.
 *
 * Revision 1.14  2005/10/19 20:34:17  mju
 * Use gre namespace instead of legacy names.
 *
 * Revision 1.13  2005/10/14 17:31:44  mju
 * GRE namespace and treeview.
 *
 * Revision 1.12  2005/06/17 16:48:19  mju
 * Add dlgControls member so can be modeless.
 *
 * Revision 1.11  2005/02/25 23:31:25  scowan
 * Added copy virtual method implementation.
 *
 * Revision 1.10  2005/02/14 21:39:56  mju
 * Use device in usesTransparency.
 *
 * Revision 1.9  2005/01/20 16:57:13  mju
 * Redesign layer type registration.
 *
 * Revision 1.8  2004/10/01 17:20:09  mju
 * Add dlgparent to assignObject.
 *
 * Revision 1.7  2004/06/28 17:53:56  mju
 * Convert to spatref system.
 *
 * Revision 1.6  2004/03/05 15:13:00  mju
 * Use virtual dtor for dispparm.
 *
 * Revision 1.5  2004/02/13 15:46:24  scowan
 * Fixed deprecated mgd items.
 *
 * Revision 1.4  2003/09/23 13:16:53  mju
 * Explicit init base in dispparm copy ctor.
 *
 * Revision 1.3  2003/09/15 13:48:59  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.2  2003/07/30 15:25:59  mju
 * Ignore private sections.
 *
 * Revision 1.1  2003/06/13 22:09:27  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_LRREGION_H
#define	INC_GRE_LRREGION_H

#ifndef	INC_GRE_LAYER_H
#include <gre/layer.h>
#endif

#ifndef	INC_GRE_LEGEND_H
#include <gre/legend.h>
#endif

namespace GRE {

class LAYER_REGION : public GRE::LAYER {
	public:

		// LAYER_REGION::DISPPARM
		class DISPPARM : public GRE::LAYER::DISPPARM {
			public:

				class DLG;

				RVC::OBJITEM m_ObjItem;
				RVC::OBJITEM m_ObjItemStyle;
				POLYSTYLE m_Style;

				DISPPARM ();
				DISPPARM (
					const DISPPARM& rhs
					): GRE::LAYER::DISPPARM(rhs) { Copy(rhs); }
				virtual ~DISPPARM (
					) { Free(); }

				DISPPARM& operator= (const DISPPARM& rhs) {
					if (this != &rhs) {
						Free();
						Copy(rhs);
						}
					return (*this);
					}

				//! Set to use specified object.
				ERRVALUE SetObject (
					const RVC::OBJITEM& objitem,		//!< Object to set
					LAYER_REGION* layer,
					bool keepsettings = false		//!< Retain current display settings
					);
					
			protected:
				virtual const SERIALIZERITEM* SerialGetItemDef (SERIALIZER& serializer) const;

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				void Copy (const DISPPARM&);
				void Free ();
				static ERRVALUE SerializerCB_StyleFileName (SERIALIZER&, const SERIALIZERITEM *, void *, SERIALIZERITEM::ACTION);

				virtual ERRVALUE v_SetObjItem (const RVC::OBJITEM& objitem, bool replace);
				virtual ERRVALUE v_Validate ();
				friend class LAYER_REGION;
				#endif //!< GENERATING_DOXYGEN_OUTPUT
			};	// End of LAYER_REGION::DISPPARM

		//! Register this layer type
		static void RegisterType (
			);

		//! Construct layer and insert into list
		explicit LAYER_REGION (
			GRE::GROUP *group,										//!< Group to contain layer
			CREATEFLAGS createflags = CREATEFLAG_None,	//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,		//!< Position in list
			GRE::LAYER *reflayer = 0								//!< Reference layer for insertion
			);

		//! Destructor.
		virtual ~LAYER_REGION ();

		//! Clear region.
		void ClearRegion (
			);

		//! Retrieve layer-specific display parameters.
		const DISPPARM& GetDispParm (
			) const { return (m_dispparm); }

		//! Set layer-specific display parameters.
		//! This is the recommended method for changing how an existing layer is to be
		//! rendered.  Validation will be performed on the specified parameters and
		//! adjustments made to the layer's copy if necessary.  Use of this method
		//! usually result in a ChangeBegin / ChangeEnd message pair, and will also
		//! generate a ChangeObject message if the spatial object used in the layer
		//! is changed.
		ERRVALUE SetDispParm (
			const DISPPARM& dispparm					//!< Parameters to set
			);

		void SetRegion (
			const REGION2D& region
			);

	protected:

		DISPPARM m_dispparm;								//!< Layer-specific display parameters

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		static void LegendMsgHandler (GRAPHICTREEITEM_MSG *msg);

	#ifdef USE_MGUI
		MGUI::DLGSHELL *m_pDlgControls;
	#endif
		GRE::LEGEND_POLYGON m_Legend;

		int DrawSetup (void*, GRE::LAYERDC&);

		//! Overrides from GRE::LAYER.
		virtual ERRVALUE v_AssignObject (const RVC::OBJITEM& objitem, MDLGPARENT dlgparent);
	#ifdef USE_MGUI
		virtual int v_ControlPanel (MDLGPARENT dlgparent,UINT32 flags = 0);
	#endif
		virtual GRE::LAYER* v_Copy (GRE::GROUP *group) const;
		virtual ERRVALUE v_Draw (GRE::LAYERDC& layerdc);
		virtual const GRE::LEGEND* v_GetLegend (ELEMTYPE ElemType);
		virtual const RVC::OBJITEM& v_GetObjItem (int component) const;
		virtual bool v_HasLegend () const;
	#ifdef USE_MGUI
		virtual ERRVALUE v_PromptObject (bool OpenCtrlPanel, MDLGPARENT dlgparent);
	#endif
		virtual const char* v_SerialGetTagName () const;
		virtual void v_SetLegendSampleFlags (MGD::STYLESAMPLEFLAGS SampleFlags);
		virtual ERRVALUE v_SetStyleObjItem (const RVC::OBJITEM& ObjItem);
	#ifdef USE_MGUI
		virtual MGUI::CTRL_TREE::ITEMHND v_TreeViewAddUpdateLegend (GRE::CTRL_TREEVIEW& TreeView, MGUI::CTRL_TREE::ITEMHND hndParent, MGUI::CTRL_TREE::ITEMHND hndInsertAfter);
	#endif
		virtual int v_UpdateExtents (bool forceupdate);
		virtual bool v_UsesTransparency (const MGD::DEVICE *pDevice = 0) const;

		LAYER_REGION (const LAYER_REGION&);
		LAYER_REGION& operator= (const LAYER_REGION&);
		
		class PRIVDRAWDATA;
		friend class PRIVDRAWDATA;
		friend class DISPPARM;

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}	// End namespace GRE

#endif	// INC_GRE_LRREGION_H
