/**
 * \file gre/uigadgetscalebar.h
 * \brief GRE::UI::GADGET_SCALEBAR definitions
 *
 * \if NODOC
 * $Id: uigadgetscalebar.h_v 1.6 2011/10/20 13:15:52 mju Exp $
 *
 * $Log: uigadgetscalebar.h_v $
 * Revision 1.6  2011/10/20 13:15:52  mju
 * Remove redundant hdrs.
 *
 * Revision 1.5  2011/03/14 15:30:26  mju
 * Add settings::get/setTextFont/Size.
 *
 * Revision 1.4  2011/03/10 16:19:19  mju
 * Add tooltip.
 *
 * Revision 1.3  2011/03/07 21:25:26  mju
 * Add settings methods for all colors.
 *
 * Revision 1.2  2011/03/07 18:56:04  mju
 * Add settings::hasDualUnits.
 *
 * Revision 1.1  2011/03/03 22:24:45  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_UIGADGETSCALEBAR_H
#define	INC_GRE_UIGADGETSCALEBAR_H

#ifndef	INC_MGUI_GADGET_H
#include <mgui/gadget.h>
#endif

#ifndef	INC_MI32_UOM_H
#include <mi32/uom.h>
#endif

#ifndef	INC_MI32_ELEMSTYL_H
#include <mi32/elemstyl.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
namespace GRE {
	class VIEW;
	class MSG;
	}
#endif

namespace GRE {
namespace UI {

//=====================================================================================================================

class GADGET_SCALEBAR : public MGUI::GADGET {
	public:

		enum STYLE {
			STYLE_UnitSingle =	0x0000,		//!< Single unit (default)
			STYLE_UnitDual =		0x8000,		//!< Dual units, cannot combine with STYLE_UnitSingle
			STYLE_UnitCombined =	0x4000,
			STYLE_LabelAbove =	0x0000,		//!< Label above bar (default)
			STYLE_LabelBelow =	0x2000,		//!< Label below bar, cannot combine with STYLE_LabelAbove
			STYLE_Box =				0,
			STYLE_IBeam =			1,
			STYLE_UBeam =			2,
			STYLE_DualCombinedBeams =	STYLE_UnitCombined | STYLE_IBeam,
			STYLE_MASK_Shape =	0x00FF
			};

		class SETTINGS {
			public:

				enum STEP {
					STEP_1_2_5 =		0,	// 1, 2, 5, 10, 20, 50, 100, 200, 500, ...
					STEP_10_25_50 =	1,	// 1, 2, 5, 10, 25, 50, 100, 250, 500, ...
					};

				SETTINGS ();
				~SETTINGS ();

				STYLE GetBarStyle () const
					{ return (m_BarStyle); }

				const COLOR& GetColorBar () const
					{ return (m_ColorBar); }

				const COLOR& GetColorFrame () const
					{ return (m_ColorFrame); }

				GADGET::PLACEMENT GetGadgetPlacement () const
					{ return (m_Placement); }

				UINT16 GetMaxLength () const
					{ return (m_MaxLength); }

				STEP GetStep () const
					{ return (m_Step); }

				const COLOR& GetTextColorBG () const
					{ return (m_TextStyle.BGColor); }

				const COLOR& GetTextColorFG () const
					{ return (m_TextStyle.FGColor); }

				const MISTRING& GetTextFont () const
					{ return (m_TextStyle.fontname); }

				UINT16 GetTextSize () const
					{ return (static_cast<UINT16>(m_TextStyle.Height)); }

				UOM::SYSTEMID GetUnitSystemId1 () const
					{ return (m_UnitSystemId1); }

				UOM::SYSTEMID GetUnitSystemId2 () const
					{ return (m_UnitSystemId2); }

				bool HasDualUnits () const;

				void IniRead (INIHANDLE inihnd, const char *inigroup);
				void IniWrite (INIHANDLE inihnd, const char *inigroup) const;

				void SetBarStyle (
					STYLE BarStyle
					) { m_BarStyle = BarStyle; }

				void SetColorBar (
					const COLOR& ColorBar
					) { m_ColorBar = ColorBar; }

				void SetColorFrame (
					const COLOR& ColorFrame
					) { m_ColorFrame = ColorFrame; }

				void SetGadgetPlacement (
					GADGET::PLACEMENT Placement
					) { m_Placement = Placement; }

				void SetMaxLength (
					UINT16 MaxLength						// Maximum bar length in pixels
					) { m_MaxLength = MaxLength; }

				void SetStep (
					STEP step
					) { m_Step = step; }

				void SetTextColorBG (
					const COLOR& color
					);

				void SetTextColorFG (
					const COLOR& color
					) { m_TextStyle.FGColor = color; }

				void SetTextFont (
					const MISTRING& FontName
					) { m_TextStyle.fontname = FontName; }

				void SetTextSize (
					UINT16 TextSize
					) { m_TextStyle.Height = TextSize; }

				const TEXTSTYLE& GetTextStyle () const
					{ return (m_TextStyle); }

				void SetUnitSystems (
					UOM::SYSTEMID UnitSystemId1,
					UOM::SYSTEMID UnitSystemId2
					) { m_UnitSystemId1 = UnitSystemId1; m_UnitSystemId2 = UnitSystemId2; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				TEXTSTYLE m_TextStyle;
				COLOR m_ColorFrame;
				COLOR m_ColorBar;
				GADGET::PLACEMENT m_Placement;
				STYLE m_BarStyle;
				UINT16 m_MaxLength;
				STEP m_Step;
				UOM::SYSTEMID m_UnitSystemId1;
				UOM::SYSTEMID m_UnitSystemId2;
				#endif
			};

		//! Contructor.
		GADGET_SCALEBAR (
			MGUI::TOOLCANVAS *pCanvas,
			VIEW *view
			);

		//! Destructor.
		~GADGET_SCALEBAR ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class DLG_SETTINGS;

		class BAR {
			public:
				enum PARTS {
					PART_Frame =	0x01,
					PART_Inside =	0x02,
					PART_Label =	0x04,
					PARTS_All =		0xFF
					};

				BAR (
					bool IsSecond
					);
				INT16 Calc (
					const SETTINGS& Settings,
					MGD::DEVICE& device,
					double scale
					);
				void CalcSample (
					const SETTINGS& Settings,
					STYLE BarStyle,
					const LRECT2D& rect
					);
				void Draw (
					const SETTINGS& Settings,
					MGD::CONTEXT *gc,
					MGUI::POINT pos,
					UINT8 parts = PARTS_All
					) const;
				UINT16 GetLengthInPixels () const
					{ return (m_LengthInPixels); }
			private:
				bool m_IsSecond;
				bool m_LabelBelow;
				STYLE m_Style;
				UINT16 m_LengthInPixels;
				INT16 m_BarCenterY;
				INT16 m_LabelBaseY;
				MISTRING m_Label;
			};

		VIEW *m_view;
		SETTINGS m_Settings;
		BAR m_Bar1;
		BAR m_Bar2;

		static void GRECB_View (const GRE::MSG *pMsg, void *vpGadget);

		void GetScaleStr (UOM::SYSTEMID UnitSystem, MISTRING& str) const;

		// GADGET overrides
		virtual MGUI::POINT v_ComputeSize ();
		virtual int v_DlgEditSettings (MDLGPARENT dlgparent);
		virtual void v_Draw (MGD::CONTEXT *gc, MGUI::CANVAS::DEVICETYPE DeviceType);
		virtual bool v_OnToolTipRequest (MGUI::CTRL_TOOLTIP& tooltip, MGUI::POINT& ptCanvas);
		#endif
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(GADGET_SCALEBAR::STYLE)
#endif

//=====================================================================================================================

}	// End namespace UI
}	// End namespace GRE

#endif	// INC_GRE_UIGADGETSCALEBAR_H
