/**
 * \file mgui/colorsel.h
 * \brief MGUI::FORM_COLORSELECTOR class definition
 *
 * \if NODOC
 * $Id: colorsel.h_v 1.17 2011/09/01 18:14:07 mju Exp $
 *
 * $Log: colorsel.h_v $
 * Revision 1.17  2011/09/01 18:14:07  mju
 * No deprecated macro.
 *
 * Revision 1.16  2011/08/24 14:44:23  mju
 * Remove unnecessary overrides.
 *
 * Revision 1.15  2009/03/10 20:06:32  mju
 * Add onPalettePaint.
 *
 * Revision 1.11  2005/06/24 17:48:54  mju
 * Support delegates.
 *
 * Revision 1.5  2002/12/02 17:13:10  mju
 * Allow palette selection.
 *
 * Revision 1.1  2002/08/13 16:39:43  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_COLORSEL_H
#define	INC_MGUI_COLORSEL_H

#ifndef	INC_MGUI_COLORBTN_H
#include <mgui/colorbtn.h>
#endif

#ifndef	INC_MGUI_EDIT_H
#include <mgui/edit.h>
#endif

#ifndef	INC_MGUI_CANVAS_H
#include <mgui/canvas.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
class COLORPALETTE;
namespace MGD {
	class CONTEXT;
	}
#endif

namespace MGUI {

//! Form for color selection.
//! This form includes a "mini-palette", a pushbutton for choosing color, and an
//! optional "edit" control for tranparency.
class FORM_COLORSELECTOR : public MGUI::FORM_COMPOSITE {
	public:

		enum FORMSTYLE {
			FORMSTYLE_Default =				0x00,
			FORMSTYLE_EditTransparency =	0x01
			};

		//! Constructor.
		FORM_COLORSELECTOR ();

		//! Destructor.
		virtual ~FORM_COLORSELECTOR ();

		//! Create the form.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,
			FORMSTYLE formstyle = FORMSTYLE_Default
			);

		//! Get current color.
		const COLOR& GetColor (
			) const { return (m_color); }

		//! Set delegate to call when color changed.
		void SetDelegateOnChangeColor (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnChangeColor = delegate; }

		//! Set color.
		void SetColor (
			const COLOR& color,
			bool notify = true
			);

		//! Set transparency factor.
		void SetTransparency (
			UINT8 transparency,					//!< 0 = opaque, 255 = invisible.
			bool notify = true
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		typedef FORM_COLORSELECTOR THISCLASS;

		class CANVAS_PALETTE : public MGUI::CANVAS {
			public:
				CANVAS_PALETTE (
					FORM_COLORSELECTOR& form
					): m_form(form) { }
			private:
				FORM_COLORSELECTOR& m_form;
				virtual void v_OnPointerButtonDown (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);
			};
		friend class CANVAS_PALETTE;

		DELEGATE_VOID_NOPARMS m_DelegateOnChangeColor;
		COLOR m_color;
		int m_palindex;
		int m_cellsize;
		int m_x0;
		int m_y0;
		CANVAS_PALETTE m_PaletteCanvas;
		CTRL_PUSHBUTTON m_BtnPalette;
		FORM_COLORBUTTON m_ColorBtn;
		FORM_EDIT_NUMBER m_TransparencyEN;
		const COLORPALETTE *m_pColorPalette;

		void DrawHighlight (int palindex, bool set);
		void DrawPalette ();
		void NotifyChangeColor ();
		void OnCmdColor ();
		void OnCmdPalette ();
		void OnEditTransparency ();
		void OnPaletteLeftDown (MGUI::POINT point);
		void OnPalettePaint (MGD::CONTEXT *gc);

		FORM_COLORSELECTOR (const FORM_COLORSELECTOR&);
		FORM_COLORSELECTOR& operator= (const FORM_COLORSELECTOR&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FORM_COLORSELECTOR::FORMSTYLE)
#endif

}		// End of MGUI namespace


#endif	// INC_MGUI_COLORSEL_H
