/**
 * \file mgui/formodbc.h
 * \brief MGUI::FORM_ODBC class definitions
 *
 * \if NODOC
 * $Id: formodbc.h_v 1.15 2011/11/29 22:28:03 mju Exp $
 *
 * $Log: formodbc.h_v $
 * Revision 1.15  2011/11/29 22:28:03  mju
 * Remove redundant hdrs.
 *
 * Revision 1.14  2007/05/07 14:21:38  mju
 * Use mistringlist hdr.
 *
 * Revision 1.10  2003/09/29 17:48:00  scowan
 * Added another odbc derived user interface dialog class.
 *
 * Revision 1.8  2003/06/05 19:10:51  scowan
 * Added oracle spatial specific form definition.
 *
 * Revision 1.3  2002/12/10 19:02:51  scowan
 * Added schema use.
 *
 * Revision 1.1  2002/12/03 17:07:10  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_FORMODBC_H
#define	INC_MGUI_FORMODBC_H

#ifndef	INC_MGUI_EDIT_H
#include <mgui/edit.h>
#endif

#ifndef	INC_MGUI_LISTBOX_H
#include <mgui/listbox.h>
#endif

#ifndef	INC_MI32_MISTRINGLIST_H
#include <mi32/mistringlist.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
namespace RVC {
	class ODBCINTERFACE;
	}
#endif


namespace MGUI {
	
//===================================================================================================================
//! Form containing ODBC Data Source selection
class FORM_ODBC_SOURCE : public MGUI::FORM_COMPOSITE {
	public:
	
		class DATASOURCEINFO {
			public:
				enum FILEUSAGE {
					FILEUSAGE_DBMS = 0,		//!< No file associated with the source
					FILEUSAGE_FILE,			//!< One file associated with the source (the table is the file)
					FILEUSAGE_CATALOG			//!< Multiple tables associated with the source (the file is a catalog of tables)
					};
			
				DATASOURCEINFO (
					) : m_IsValid(true) {}
				
				const MISTRING& GetDriver (
					) const { return (m_Driver); } 
			
				const MISTRING& GetName (
					) const { return (m_Name); } 
			
				MISTRING GetFileConnectTypeStr (
					) const;
						
				MISTRING GetFileExtns (
					) const;

				FILEUSAGE GetFileUsage (
					) const;
					
				bool IsValid (
					) const { return (m_IsValid); }
				
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				MISTRING m_Name;
				MISTRING m_Driver;
				MISTRING m_Attrib;
				bool m_IsValid;
				friend class FORM_ODBC_SOURCE;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};
	
		//! Constructor.
		FORM_ODBC_SOURCE (
			);

		//! Destructor.
		virtual ~FORM_ODBC_SOURCE (
			);

		//! Create
		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,	//!< Parent pane
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedWidth
			);
			
		const DATASOURCEINFO& GetDataSource (
			) const;
			
	protected:
	
		virtual bool v_DataSourceFilter (const MISTRING& Driver);
		virtual void v_OnDataSourceSelect (const DATASOURCEINFO& DataSource);
		virtual void v_OnNewSourceFile (const DATASOURCEINFO& DataSource);
	
		ERRVALUE FillDataSourceList ();
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		CTRL_GROUPBOX m_DataSourceBox;
		CTRL_LISTBOX m_DataSource;
		LAYOUT_PANE m_DSNButtons;
		CTRL_PUSHBUTTON m_CtrlPanel;
		CTRL_PUSHBUTTON m_NewFileBtn;
		MILIST<DATASOURCEINFO> m_DataSourceList;
		MILIST<DATASOURCEINFO>::ITERATOR m_CurDataSource;
	
		void OnCtrlPanelPressed ();
		void OnDataSourceChange ();
		void OnNewSourceFile ();
		
		FORM_ODBC_SOURCE (const FORM_ODBC_SOURCE&);
		FORM_ODBC_SOURCE& operator= (const FORM_ODBC_SOURCE&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	
	};
	
//===================================================================================================================
//! Form containing ODBC Data Source and table selection
class FORM_ODBC : public MGUI::FORM_COMPOSITE {
	public:
	
		class CONNECTION_ITEM {
			public:
				CONNECTION_ITEM () : m_FileUsage(FORM_ODBC_SOURCE::DATASOURCEINFO::FILEUSAGE_DBMS) {}
			
				//! Get the Connection string of the selected data source
				//! @return Connection string or an empty string if none selected
				const MISTRING& GetConnectString (
					) const { return (m_ConnectString); }

				//! Get the file usage for the connection
				//! @return FILEUSAGE					
				FORM_ODBC_SOURCE::DATASOURCEINFO::FILEUSAGE GetFileUsage (
					) const { return (m_FileUsage); }
					
				//! Get the Data Source Name (DSN) of the selected data source
				//! @return Selected DSN or an empty string if none selected
				const MISTRING& GetDataSourceName (
					) const { return (m_DataSourceName); }
					
				const MISTRINGLIST& GetTableNameList (
					) const { return (m_SelTableNameList); }
					
				const MISTRINGLIST& GetTableDescList (
					) const { return (m_SelTableDescList); }
					
				const MISTRINGLIST& GetTableSchemaList (
					) const { return (m_SelTableSchemaList); }
					
				bool IsTableSelected (
					) const { return (!m_SelTableNameList.IsEmpty()); }
			
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				MISTRING m_DataSourceName;
				MISTRING m_ConnectString;
				MISTRINGLIST m_SelTableNameList;
				MISTRINGLIST m_SelTableDescList;
				MISTRINGLIST m_SelTableSchemaList;
				FORM_ODBC_SOURCE::DATASOURCEINFO::FILEUSAGE m_FileUsage;
				
				friend class FORM_ODBC;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};
	

		//! Constructor.
		FORM_ODBC (
			);

		//! Destructor.
		virtual ~FORM_ODBC (
			);

		//! Create
		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,	//!< Parent pane
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_Expand
			);
			
		void DisplayMessage (
			const MISTRING& Message
			);

		//! Get the list of selected sources and tables from the FORM			
		const MILIST<CONNECTION_ITEM>& GetConnectionList (
			) const { return (m_SelectedList); } 
			
	protected:
	
		virtual bool DataSourceFilter (const MISTRING& Driver);
		virtual ERRVALUE OnDataSourceSelected (const RVC::ODBCINTERFACE& ODBCInt);
		virtual void OnTableSelected ();
		virtual bool TableFilter (const RVC::ODBCINTERFACE& ODBCInt, const MISTRING& TableName, const MISTRING& SchemaName);
		
		void RefreshTableList ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class ODBC_SOURCE : public FORM_ODBC_SOURCE {
			public:
				ODBC_SOURCE (
					FORM_ODBC& OdbcForm
					) : FORM_ODBC_SOURCE(), m_OdbcForm(OdbcForm) {}
					
				void FillDataSourceList (
					);
			
			private:
				FORM_ODBC& m_OdbcForm;
			
				virtual bool v_DataSourceFilter (const MISTRING& Driver);
				virtual void v_OnDataSourceSelect (const FORM_ODBC_SOURCE::DATASOURCEINFO& DataSource);
				virtual void v_OnNewSourceFile (const FORM_ODBC_SOURCE::DATASOURCEINFO& DataSource);
			
			};
		friend class ODBC_SOURCE;
	
		LAYOUT_PANE m_ListForm;
		RVC::ODBCINTERFACE* m_ODBCInterface;
		ODBC_SOURCE m_ODBCSource;

		//! Data source selection list data		
		MILIST<CONNECTION_ITEM> m_SelectedList;
		MILIST<CONNECTION_ITEM>::ITERATOR m_CurSourceItem;
		
		CTRL_GROUPBOX m_TableListBox;
		CTRL_LISTBOX m_TableList;
		MISTRINGLIST m_TableNameList;
		MISTRINGLIST m_TableDescList;
		MISTRINGLIST m_TableSchemaList;
		
		LAYOUT_PANE m_AddRemoveForm;
		CTRL_PUSHBUTTON m_AddTable;
		CTRL_PUSHBUTTON m_RemoveTable;
		
		CTRL_GROUPBOX m_SelectedTableBox;
		CTRL_LISTBOX m_SelectedTables;
		MISTRINGLIST m_SelConList;
		CTRL_TEXT m_DescText;

		void OnAddTable ();
		void OnDataSourceChange (const FORM_ODBC_SOURCE::DATASOURCEINFO& DataSource);
		void OnNewSourceFile (const FORM_ODBC_SOURCE::DATASOURCEINFO& DataSource);
		void OnRemoveTable ();
		void OnSelectedTableChange ();
		void OnTableListChange ();
		
		ERRVALUE EstablishNewConnection ();
	
		FORM_ODBC (const FORM_ODBC&);
		FORM_ODBC& operator= (const FORM_ODBC&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	
	};
	

//===================================================================================================================

class FORM_ODBC_ORACLE_SPATIAL : public MGUI::FORM_ODBC {
	public:
		//! Constructor	
		FORM_ODBC_ORACLE_SPATIAL (
			);
			
		//! Destructor
		~FORM_ODBC_ORACLE_SPATIAL (
			);
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual ERRVALUE OnDataSourceSelected (const RVC::ODBCINTERFACE& ODBCInt);
		virtual bool DataSourceFilter (const MISTRING& dsn);
		virtual bool TableFilter (const RVC::ODBCINTERFACE& ODBCInt, const MISTRING& TableName, const MISTRING& SchemaName);
				
		MISTRING m_TestName;
		MISTRINGLIST m_ValidSchemaList;
		MISTRINGLIST m_ValidTableList;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

	
class FORM_ODBC_PRIMARY : public MGUI::FORM_COMPOSITE {
	public:
		//! Constructor	
		FORM_ODBC_PRIMARY (
			);
			
		//! Destructor
		~FORM_ODBC_PRIMARY (
			);
			
		//! Check if the table selected can be random accessed
		bool CanRandomAccess (
			const MISTRING& TableName,
			const MISTRING& SchemaName
			) const { return (m_PrimaryODBC.CanRandomAccess(TableName, SchemaName)); }
			
		//! Create
		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,	//!< Parent pane
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_Expand
			);
			
		//! Get the list of selected sources and tables from the FORM			
		const MILIST<FORM_ODBC::CONNECTION_ITEM>& GetConnectionList (
			) const { return (m_PrimaryODBC.GetConnectionList()); } 
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIMARY_ODBC : public FORM_ODBC {
			public:
				PRIMARY_ODBC () : FORM_ODBC(), m_PrimaryOnly(true) {}
				~PRIMARY_ODBC () {}
				
				bool CanRandomAccess (
					const MISTRING& TableName,
					const MISTRING& SchemaName
					) const;
			
				void SetPrimaryFlag (
					bool value
					);
		
			private:
				MISTRINGLIST m_ValidSchemaList;
				MISTRINGLIST m_ValidTableList;
				bool m_PrimaryOnly;
				
				virtual ERRVALUE OnDataSourceSelected (const RVC::ODBCINTERFACE& ODBCInt);
				virtual bool TableFilter (const RVC::ODBCINTERFACE& ODBCInt, const MISTRING& TableName, const MISTRING& SchemaName);
			};
		
		PRIMARY_ODBC m_PrimaryODBC;
		CTRL_TOGGLEBUTTON m_PrimaryOnly;
		
		void OnPrimaryOnly ();
		
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}		// End of MGUI namespace

#endif	// INC_MGUI_FORMODBC_H
