/**
 * \file mgui/formquerybuilder.h
 * \brief MGUI::FORM_QUERY_BUILDER definitions
 *
 * \if NODOC
 * $Id: formquerybuilder.h_v 1.1 2010/01/27 17:37:55 mju Exp $
 *
 * $Log: formquerybuilder.h_v $
 * Revision 1.1  2010/01/27 17:37:55  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_FORMQUERYBUILDER_H
#define	INC_MGUI_FORMQUERYBUILDER_H

#ifndef	INC_MGUI_FORM_H
#include <mgui/form.h>
#endif

#ifndef	INC_MI32_ELEMTYPE_H
#include <mi32/elemtype.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
namespace RVC {
	class OBJITEM;
	}
#endif

namespace MGUI {

//=====================================================================================================================

//! Form for building query script using geometric object database.
class FORM_QUERY_BUILDER : public MGUI::FORM_COMPOSITE {
	public:

		//! Constructor.
		FORM_QUERY_BUILDER ();

		//! Destructor.
		~FORM_QUERY_BUILDER ();

		//! Create form.
		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane to contain form
			const RVC::OBJITEM& ObjItem,				//!< Geospatial object location 
			ELEMTYPE ElemType,							//!< Element type
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_Expand
			);

		//! Get query script string.
		const MISTRING& GetScript (
			bool AddLineBreaks = false
			) const;

		//! Determine if has changed.
		bool IsChanged () const;

		//! Determine if current query is valid.
		bool IsValid () const;

		//! Reset "changed" status.
		void ResetChanged ();

		//! Set delegate to call when any change to script is made.
		void SetDelegateOnChange (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set delegate to call when user chooses table.
		//! Typically used to show table window.
		void SetDelegateOnChooseTable (
			DELEGATE_VOID_INT32 delegate
			);

		//! Set delegate to call to display status message.
		void SetDelegateOnSetMessage (
			DELEGATE_VOID_MISTRING delegate
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		FORM_QUERY_BUILDER (const FORM_QUERY_BUILDER&);
		FORM_QUERY_BUILDER& operator= (const FORM_QUERY_BUILDER&);
		#endif
	};

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_FORMQUERYBUILDER_H
