/*****************************************************************************
 * \file mi32/dbedit.h
 * \brief DBEDIT class definitions.
 *
 *	\if NODOC
 *	$Id: dbedit.h_v 1.78 2012/09/25 19:23:28 mju Exp $
 *
 *	$Log: dbedit.h_v $
 *	Revision 1.78  2012/09/25 19:23:28  mju
 *	Add getDisplayPath.
 *
 *	Revision 1.77  2012/06/05 15:00:28  mju
 *	Add getCreateFlags.
 *
 *	Revision 1.76  2011/10/24 20:34:50  mju
 *	Remove redundant hdrs.
 *
 *	Revision 1.75  2011/09/06 21:26:52  mju
 *	Rename macro.
 *
 *	Revision 1.74  2010/08/19 17:39:38  mju
 *	Use elemnum and unsignedset.
 *
 *	Revision 1.73  2010/05/13 14:04:34  mju
 *	Fix warning.
 *
 *	Revision 1.72  2009/08/11 21:54:09  mju
 *	Add isFormOpen.
 *
 *	Revision 1.71  2009/05/12 16:29:46  mju
 *	Rename flag to noSwitchTableView.
 *
 *	Revision 1.70  2008/09/26 21:15:41  mju
 *	Remove activeElemList, just use activeElemNum.
 *
 *	Revision 1.69  2008/09/22 19:01:20  mju
 *	Update for dbrecord viewers.
 *
 *	Revision 1.68  2008/08/27 15:25:24  mju
 *	Remove unneeded cdbeditfuncs decl.
 *
 *	Revision 1.67  2008/07/15 21:57:48  mju
 *	Add canMakeTableImplied.
 *
 *	Revision 1.66  2008/01/04 22:12:02  scowan
 *	Added tableCreateDefinition static method.
 *
 *	Revision 1.65  2007/11/12 15:47:40  mju
 *	Don't use mstatus hdr as nothing in this hdr uses any status stuff.
 *
 *	Revision 1.63  2006/12/07 16:13:34  dwilliss
 *	some changes for images in blobs
 *
 *	Revision 1.43  2005/10/06 17:28:02  mju
 *	Add default for openTreeView.
 *
 *	Revision 1.41  2005/08/08 14:04:41  dwilliss
 *	Changed the type of a parameter to satisfy gcc
 *
 *	Revision 1.39  2005/07/13 15:49:59  dwilliss
 *	Make it possible to eliminate write-lock-as-needed mode
 *
 *	Revision 1.38  2005/03/31 16:57:10  fileserver!dwilliss
 *	Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 *	Revision 1.36  2004/05/13 15:39:07  dwilliss
 *	Table creation wizard done
 *
 *	Revision 1.35  2004/05/10 20:57:31  dwilliss
 *	Added a fieldinfo to TableCreate so you can create with initial fields
 *
 *	Revision 1.34  2004/04/12 16:42:49  dwilliss
 *	Changed style editing to use MGUI and RVC interface classes
 *
 *	Revision 1.32  2003/10/13 20:45:06  scowan
 *	Replaced callback queue pointer with a simple array.
 *	Removed unused members.
 *
 *	Revision 1.29  2002/09/17 14:15:30  dwilliss
 *	Added new SetStyleObject method that takes OBJITEM
 *
 * Revision 1.1  1999/04/07  22:43:25  dwilliss
 * Initial revision
 *
 *	\endif
 ****************************************************************************/

#ifndef INC_MI32_DBEDIT_H
#define INC_MI32_DBEDIT_H

#ifndef INC_MI32_LONGLIST_H
	#include <mi32/longlist.h>
#endif

#ifndef INC_MI32_ELEMSTYL_H
	#include <mi32/elemstyl.h>
#endif

#ifndef INC_MI32_MICALLBACKLIST_H
	#include <mi32/micallbacklist.h>
#endif

#ifndef INC_RVC_STYLE_H
	#include <rvc/style.h>
#endif

#ifndef INC_RVC_DBELEM_H
	#include <rvc/dbelem.h>
#endif


//=====================================================================================================================

#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations
class DBEDIT;
class DBEDIT_TDATA;			// Table data
class DBEDIT_TINSTANCE;		// Table instance
class DBEDIT_STYLEDIALOG;		// Style Data
struct DBEDIT_FIELDOPT;		// Field options

struct RECSTYLESAMPLE;
namespace RVC {
	class DBASE;
	class DBASE_ELEMENT;
	namespace UI {
		class DLG_DB_BASE;
		class FORM_DB_RECORD;
		}
	}
namespace MGUI {
	class LAYOUT_PANE_BASE;
	}
#endif	// GENERATING_DOXYGEN_OUTPUT

//=====================================================================================================================

//! Message sent to DBEDIT Callbacks
struct DBEDIT_MSG : public MICALLBACKLIST::ANYCALLBACKSTRUCT {

	enum REASON {
		REASON_ALL					= 0x00000000,
		REASON_TableOpen			= 0x00000001,
		REASON_TableClose			= 0x00000002,
		REASON_FormOpen			= 0x00000004,
		REASON_FormClose			= 0x00000008,
		REASON_Destroy				= 0x80000000,		//!< DBEDIT is being destroied
		REASON_Modify				= 0x04000000,
		REASON_SelectElements	= 0x00000010,
		REASON_UnselectElements	= 0x00000020,
		REASON_ToggleElements	= 0x00000040,
		REASON_SelectOnly			= 0x00000080,
		REASON_Elements			= 0x000000F0,		//!<  any of the 4 above 
		REASON_FormObjectCreate	= 0x00000100,		//!<  A form object has been created under this database 
		REASON_FormObjectDestroy= 0x00000200,		//!<  A form object under this database has been destroyed 
		REASON_AutoComplete		= 0x00000400
		};

	//! Constructor.
	DBEDIT_MSG (
		DBEDIT* p_DBEdit,
		REASON r	
		) :
		ANYCALLBACKSTRUCT(r),
		DBEdit(p_DBEdit),
		TableNum(0),
		ElemType(ELEMTYPE_Empty),
		Record(0)
		{ }

	RVC::OBJITEM ObjItem;		//!< ObjItem of the form (for FormOpen/FormClose only)
	DBEDIT* DBEdit;				//!< Pointer to the DBEDIT
	ELEMTYPE ElemType;
	INT32 TableNum;				//!< Table Number
	INT32 Record;
	SIMPLE_ARRAY<INT32> Records;
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(DBEDIT_MSG::REASON)
#endif

//=====================================================================================================================

//! Manager for user interface windows associated with databases and tables.
class DBEDIT {
	public:
		
		#ifndef GENERATING_DOXYGEN_OUTPUT
		enum MODE {
			MODE_SingleRecord	= 0,
			MODE_UserDefined,
			MODE_TabularView,
			MODE_NumModes
			};
		#endif

		//! Flags for database table viewer.
		enum VIEWFLAGS {
			VIEWFLAG_None =						0,
			VIEWFLAG_ViewActive =				0x00000001,	//!< Default to viewing active
			VIEWFLAG_ViewSelected =				0x00000002,	//!< Default to viewing Selected
			VIEWFLAG_ViewAll =					0x00000004,	//!< Default to viewing all records
			VIEWFLAG_NoChangeViewMode =		0x00000008,	//!< Don't let user change view mode (attached/selected/all)
			VIEWFLAG_NoSwitchTableView =		0x00000010,	//!< Don't allow user to switch between single-record/tabular view
			VIEWFLAG_NoRecordSelection =		0x00000020,	//!< Omit record selection buttons
			VIEWFLAG_NoAttachmentIndicator =	0x00000040,	//!< Omit record attachment indicators
			VIEWFLAG_NoRowControls =			(VIEWFLAG_NoRecordSelection|VIEWFLAG_NoAttachmentIndicator),
			VIEWFLAG_NoNewRecord =				0x00000080,	//!< Don't allow user to add new record
			VIEWFLAG_NoDropTable =				0x00000100,	//!< Don't allow user to "drop" table
			VIEWFLAG_NoEditDefn =				0x00000200,	//!< Don't allow user to edit table definition
			VIEWFLAG_TabularView =				0x00000400,	//!< Use tabular view
			VIEWFLAG_NoElementSelection =		0x00000800,	//!< Leave off element selection button row
			VIEWFLAG_NoCreateIfNotExist =		0x00001000,	//!< For MxdDatabaseEditorGetTableInstanceHandle()
			VIEWFLAG_SingleRecordView =		0x00002000	//!< Default to single record view
			};

		enum LABELCOLOR {
			LABELCOLOR_Normal	= 0,
			LABELCOLOR_Computed,
			LABELCOLOR_PK,
			LABELCOLOR_FK
			};

		enum CREATEFLAGS {
			CREATEFLAG_None =							0x00,
			CREATEFLAG_NoEdit =						0x01,	//!< Disallow any editing
			CREATEFLAG_AllowEditInTNTatlas =		0x20	//!< Let user edit data even in TNTatlas (for sketch layers)
			};

		struct EVENT {
			EVENT();
			EVENT(const EVENT& rhs);
			INT32 reason;
			INT32 table;
			INT32 record;
			UINT32 subreason;
			};

		static RVC::UI::DLG_DB_BASE* DlgConstructTabularView (
			DBEDIT_TINSTANCE *idata
			);

		//! Install SML interface.
		static void SML_Install (
			SML::CONTEXT *context
			);

		//! Popup a table creation dialog.
		//! This pops up the "New Table" dialog to let the user design a new table.
		static ERRVALUE TableCreateDefinition (
			MDLGPARENT parent,
			RVC::OBJTYPE DBaseObjType,
			RVC::DBTABLE::MAKEPARMS& MakeParms,				//!< Filled in MakeParms, RETURNED
			RVC::DESCRIPTOR& Descriptor						//!< Filled in Descriptor, RETURNED
			);

		//! Constructor.
		//! Use Create() after construction to complete setup.
		DBEDIT ();

		//! Destructor.
		~DBEDIT();

		//! Add a callback.
		ERRVALUE CallbackAdd (
			void (*func)(const DBEDIT_MSG *, void *),
			void *data,
			DBEDIT_MSG::REASON reason
			);

		//! Remove a callback.
		void CallbackRemove (
			void (*func)(const DBEDIT_MSG *, void *),
			void *data,
			DBEDIT_MSG::REASON reason
			);

		//! Call callbacks.
		//!
		//! This should be private.  Only used by friend classes.
		void CallCallbacks (
			DBEDIT_MSG& cbs
			);

		//! Determine if allowed to make implied one-to-one table.
		bool CanMakeTableImplied (
			) const;

		//! Close everything, but don't destroy anything.
		int Close ();

		//! Setup database editor manager with associated database.
		int Create (
			MDLGPARENT parent, 
			const RVC::OBJITEM& ParentObjItem,
			ELEMTYPE elemtype,		//!< Element type of the database to open (ELEMTYPE_Record if pinode is of an OTYPE_DBASE)
			CREATEFLAGS flags = CREATEFLAG_None,
			const RVC::CREDENTIALS* Credentials = 0
			);

		//! Suspend database callbacks.  
		//!
		//! Unless ReallyIgnore is set, callbacks which
		//! happen while suspended will be queued and processed when resumed.
		//! Suspend/Resumes are refcounted.
		void DBCallbackSuspend (
			bool ReallyIgnore,
			bool Totally = false					//!< If true, throw out everything - Use with caution
			);

		//! Resume database callback handling.  
		//!
		//! If any callbacks happened while
		//!	suspended (and we're not still suspended) they will be processed.
		void DBCallbackResume (
			bool ReallyIgnore,
			bool Totally = false					//!< If true, throw out everything - Use with caution
			);

	#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Find records.
		//!
		//! Shouldn't this be private?  Should only be called by friend classes.
		int FindRecords (
			int active
			);
	#endif

		//! Force all unwritten records to be written.
		int Flush ();			

		//! Close a form.
		int FormClose (
			const RVC::OBJITEM& ObjItem
			);

		//! Delete a form.
		int FormDrop (
			MDLGPARENT dlgparent,					//!< Parent for "Are you sure" dialog
			const RVC::OBJITEM& ObjItem
			);

		//! Create a new, blank form.
		int FormNew (
			MDLGPARENT dlgparent,					//!< Parent for dialogs
			VIEWFLAGS flags = VIEWFLAG_None,
			const RVC::CREDENTIALS* Credentials = 0
			);

		//! Open a form.
		int FormOpen (
			const RVC::OBJITEM& ObjItem,
			VIEWFLAGS flags = VIEWFLAG_None,
			const RVC::CREDENTIALS* Credentials = 0
			);

		//! Create a generic single record view form for a given table.
		ERRVALUE GenViewCreate (
			MGUI::LAYOUT_PANE_BASE& ParentPane,			//!< Parent form form
			INT32 TableNum,									//!< Table number to create view for
			RVC::UI::FORM_DB_RECORD **ppFormRet = 0,	//!< Pointer to form returned
			DBEDIT::VIEWFLAGS flags = VIEWFLAG_None
			);

		//! Get the active element number.
		//!
		//! @return ELEMENTNUM_None if none active.
		INT32 GetActiveElement (
			) const {
			return (m_ActiveElemNum);
			}

		INT32 GetBaseTableNum (
			) const { return (m_BaseTable); }

		CREATEFLAGS GetCreateFlags () const
			{ return (m_flags); }

		const RVC::CREDENTIALS* GetCredentials (
			) const { return (m_Credentials); }

		RVC::DBASE_ELEMENT* GetDBase (
			) const {
			return m_DBase;
			}
		
		//! Used by database relationships dialog to make sure there's only one instance of itself.
		RVC::DBASE::OBSERVER *GetDBRelateHandle() const {
			return (m_DBRelateHandle);
			}

		//! Get display path string for database.
		//! String will be of the form [ObjectName] / [DatabaseName] or [ObjectName].
		MISTRING GetDisplayPath () const;

		//! Get parent for dialogs managed by DBEDIT instance.
		MDLGPARENT GetDlgParent (
			) const { return (m_dlgparent); }

		//! Get element list.
		const LONGLIST& GetElemList (
			) const { return (m_ElemList); }

		//! Get element type for this database.
		ELEMTYPE GetElemType (
			) const { return (m_ElemType); }

		//! Get the color to use for a label from the DBFIELDINFO.flags.
		void GetLabelColorFromFlags (
			RVC::DBTABLE::FIELDINFO::FLAGS fieldflags,
			COLOR& color
			);

		//! Get the color to use for a label given a DBEDIT::LABELCOLOR.
		void GetLabelColor (
			LABELCOLOR colornum,
			COLOR& color
			);

		//! Get the number of tables open.
		int GetNumOpen ();

		//! Get the OBJITEM of the actual database
		const RVC::OBJITEM& GetObjItem (
			) const { return m_ObjItem; }

		//! Get the OBJITEM passed to create.
		//! This is usually the OBJITEM of the parent except for main level
		//! databases, in which case it's the database's OBJITEM
		const RVC::OBJITEM& GetParentObjItem (
			) const { return m_ParentObjItem; }

		//! Get the widget used as a parent for any dialogs we create.
		MDLGPARENT GetParent (
			) const { 
			return m_dlgparent; 
			}

		//! Open the style object associated with a database.
		//!
		//!	This is for use by DBEDIT_TDATA only (?).
		bool GetStyleObject (
			RVC::STYLE& StyleObj
			);

		//! Get the DBEDIT_TDATA for a given table.
		inline DBEDIT_TDATA * GetTableData (
			int table								//!< Get internal table data
			) const {
			if (table == -2) table = -1;	// Cope with internal node table
			++table;		// To account for table -1
			if (table < 0 || static_cast<UINT32>(table) >= m_TableData.GetNumItems()) return (NULL);
			return (m_TableData[table]);
			}	

		//! Returns the instance handle for a given table and open mode.
		//!
		//!	Will create one unless passed the flag DBVIEW_NoCreateIfNotExist.
		DBEDIT_TINSTANCE * GetTableInstanceHandle (
			INT32 TableNum,						//!< Table number to look for
			VIEWFLAGS flags							//!< DBVIEW_TabularView or 0
			);

		bool HasCallbacks (
			UINT32 reason
			) const { return (m_CBList.HasCallbacks(reason)); }

		//! Determine if Create() has been done yet.
		bool IsCreated (
			) const {
			return (m_DBase != 0);
			}

		//! Determine if form is open.
		bool IsFormOpen (
			const RVC::OBJITEM& ObjItemForm
			);

		//! Returns true if this database is not under a vector or anything.
		int IsMainLevel (
			) const {
			return (m_DBase != 0 && m_DBase->GetElemType() == ELEMTYPE_Empty);
			}

		//! Returns true if the database is being used for pin-mapping.
		//!
		//! i.e., we've called SetRecord or SetElement with an ELEMTYPE
		//! of ELEMTYPE_Record.
		int IsPinMapping (
			) const {
			return (m_pin_mapping);
			}

		//! Returns 1 if the requested table is open in some way.
		int IsTableOpen (
			int TableNum							//!< Table to check for
			);

		//-----------------------------------------------
		//! Open a table in single record view as a modal dialog.
		//!
		//! @return Record number or error (May be EUserCancel).
		int ModalSingleRecordView (
			INT32 TableNum, 						//!< Table number to open
			INT32 RecordNum,						//!< Record to edit (or RECORD_New)
			RVC::DBTABLE::RECORD* InitVal,	//!< Initial record values (can be null)
			MDLGPARENT dlgparent,				//!< Parent to use for dialog
			UINT32 flags = 0						//!< Not used
			);

		//! Add an element to the multi-Select list.
		int MultiSelectAddElement (
			INT32 ElemNum 							//!< Element number or ELEMENTNUM_None
			);

		//! Remove an element from the multi-Select list.
		int MultiSelectRemoveElement (
			INT32 ElemNum							//!< Element to remove from the selected element set
			);

		//! Start adding elements to the element list, clearing previous list.
		//!
		//! SetElements() is the prefered method, but isn't well suited
		//! to CAD.  Call MultiSelectStart, MultiSelectAddElement as many
		//! times as you need, then MultiSelectEnd.
		ERRVALUE MultiSelectStart (
			int BaseTable = 0						//!< Only needed if for ELEMTYPE_Record
			);

		//! Finish a multiple element selection.
		ERRVALUE MultiSelectEnd (
			bool UpdateDialogs = true
			);

		void OnEvent(const DBEDIT::EVENT& event, bool bDeferable = true);

	#ifdef X_NATIVE
		//! Open a tree view showing table relationships.
		//! Currently only defined for X
		ERRVALUE OpenTreeView (
			UINT32 flags = 0		//!< Old MxdFLAG_Flags
			);
	#endif

		//! Notify the database editor that attachments to active element may have changed. 
		//!
		//! (Should be unnecessary now that we have
		//! database callbacks to deal with all this, but you can totally
		//! suspend the callbacks).
		int RefreshActiveElementAttachments (
			bool UpdateDialogs = true
			);

		//! Set the "active" element.
		ERRVALUE SetActiveElement (
			ELEMNUM ElemNum, 						//!< Element number or ELEMENTNUM_None
			INT32 BaseTable = 0					//!< Table to use if this is ELEMTYPE_RECORD
			) { return (SetActiveElementLow(ElemNum,true,BaseTable)); }

		//! Set the base table number for PinMapping.
		void SetBaseTable (
			int table								//!< Table number to use
			);

		//! Used by database relationships dialog to make sure there's only one instance of itself.
		void SetDBRelateHandle (
			RVC::DBASE::OBSERVER* handle
			) {
			m_DBRelateHandle = handle;
			}

		//! Set the list of selected elements using a bit array.
		ERRVALUE SetElements (
			INT32 BaseTable,						//!< Base table number
			const UNSIGNEDSET& ElemSet,		//!< Selected elements
			ELEMNUM ActiveElemNum				//!< Active element number
			);

		//! Set widget to use as parent for all dialogs.
		void SetParent (
			MDLGPARENT parent						//!< Widget to use as a parent
			) {
			m_dlgparent = parent; 
			}

		//! Set the style number to edit in the style editor dialog.
		//!
		//! Also remembers internally which record this was for.
		//! Returns without doing anything if style editor isn't open.
		int SetStyleNumber (
			int StyleNum,							//!< Style number to edit
			int RecNum								//!< Record number in Style Assignment Table
			);

		//! Set the style object to use for style samples.
		int SetStyleObject (
			const RVC::OBJITEM& StyleObjItem,				//!< Style object to use
			const RVC::OBJECTNAME& StyleTableName,			//!< Name of style table
			const void *dftstyle					//!< Default style appropriate to the database type
			);

		//! Open the style editor dialog.
		int StyleDialogOpen (
			MDLGPARENT parent						//!< Parent for the dialog
			);

		//! Close the style dialog.
		//!
		//! Actually does nothing.
		int StyleDialogClose (
			);

		//! Popup a table creation dialog.
		//! This pops up the "New Table" dialog to let the user design a new table.
		//!
		//! The MakeParms will be used for defaults.
		int TableCreate (
			MDLGPARENT parent,					//!< Parent for dialogs
			const RVC::DBTABLE::MAKEPARMS& MakeParms,				//!< Defaults for new table 
			const RVC::DESCRIPTOR* Descriptor = 0		//!< Initial descriptor (can be 0 if no default name)
			);

		//! This uses a wizard to walk the user through the first few steps of
		//! creating a table.
		int TableCreate (
			MDLGPARENT parent					//!< Parent for dialogs
			);

		//! Drop (delete) a table.
		int TableDrop (
			MDLGPARENT parent, 					//!< Parent for "Are you sure" dialog
			int TableNum							//!< Table number to drop
			);

		//! popup a dialog to allow the user to edit the table's definition.
		int TableEditDefinition (
			MDLGPARENT parent, 					//!< Parent for "Are you sure" dialog
			int TableNum							//!< Table number to drop
			);

		//! popup a dialog to allow the user to edit the table's Access Control List
		ERRVALUE TableEditACL (
			MDLGPARENT parent, 					//!< Parent for "Are you sure" dialog
			int TableNum							//!< Table number to drop
			);

		//! Open a table window in DBVIEW_TabularView mode or (default) Single Record View.
		int TableOpen (
			INT32 TableNum,						//!< Table number to open
			VIEWFLAGS flags = VIEWFLAG_None	//!< flags
			);

		//! Close all instances of a given table.
		int TableClose (
			INT32 TableNum							//!< Table number to close (all instances)
			);

		//! Destroy table instance (just the dialog, not the underlying table).
		int TableDestroy (
			int TableNum							//!< Table number to destroy
			);

		//! Force all open table windows to update.
		int UpdateDialogs ();

		//! Similar to DBCallbackResume, but if resuming (it's refcounted so may these can be nested), this will check to see if we think we need to update everything and do so.
		//! 
		//! In a perfect world, this should never
		//!	be the case.  CallbackResume will handle it all for us.  
		int UpdateResume (
			);

		//! Just calls DBCallbackSuspend().
		void UpdateSuspend();

		#ifndef GENERATING_DOXYGEN_OUTPUT
		// This is sort of private.  It constructs a SPATMOD::IMAGE::SOURCE
		// from an image stored in a BLOB field.  The reason it's here instead
		// of somewhere public is that it has to keep track of a temp file which
		// it will delete when it can.
		// Eventually we'll have a SOURCE_JPEG and you won't need a special 
		// function anyway.
		#endif

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		class OBSERVER : public RVC::DBASE::OBSERVER {
			public:
				OBSERVER (
					DBEDIT* ddata
					) : 
					RVC::DBASE::OBSERVER(*ddata->GetDBase()),
					m_ddata(ddata) {}

				virtual ~OBSERVER() {}

				virtual void OnDBListAdd(ELEMTYPE ElemType, INT32 ElemNum);
				virtual void OnDBListChanged(ELEMTYPE ElemType, INT32 ElemNum);
				virtual void OnStructChanged(INT32 TableNum, STRUCTCHANGE SubReason);
				virtual void OnTableAdd(INT32 TableNum);
				virtual void OnTableDropBegin(INT32 TableNum);
				virtual void OnTableDropEnd(INT32 TableNum);
				virtual void OnBulkModifyBegin();
				virtual void OnBulkModifyEnd();
			private:
				DBEDIT* m_ddata;
			};

		MDLGPARENT m_dlgparent;
		CREATEFLAGS m_flags;
		bool m_is_main_level;
		bool m_pin_mapping;
		bool m_multi_select;
		bool m_ReallyIgnoreCallback_DamnIt;
		bool m_NeedUpdate;

		RVC::OBJITEM m_ObjItem;
		RVC::OBJITEM m_ParentObjItem;
		RVC::DBASE_ELEMENT* m_DBase;

		INT32 m_ActiveElemNum;							//!< "Active" element number 
		LONGLIST m_ElemList;
		ELEMTYPE m_ElemType;
		INT32 m_BaseTable;						//!<  For Pin-Mapping, record # is in Element 
		DBEDIT_TDATA **m_formtdata;
		DBEDIT_TDATA *m_OrphanInstances;		//!<  Instances with out a table 
		int m_numtdatas, m_numforms;
		int m_CreatingTable;						//!<  Table being created by workproc 
		MICALLBACKLIST m_CBList;
		struct STYLE {
			RVC::OBJITEM m_StyleObjItem;		//!< 	Style object filename 
			int m_table;							//!<  Table number of StyleAssignmentTable 
			bool m_bDefaultKnown;
			DBEDIT_STYLEDIALOG *m_dlg;			//!<  style editor dialog 
			MULTISTYLE m_dft;
				
			STYLE (
				) :
				m_table(0),
				m_bDefaultKnown(false),
				m_dlg(0)
				{ }
			} m_Style;
		RVC::DBASE::OBSERVER *m_DBRelateHandle;
		int m_IgnoreCallback;
		void *m_ClassPrivData[MODE_NumModes];

		MILIST<EVENT> m_DBEventQueue;
		bool m_bDrainingQueue;
		int m_ReallyIgnoreCallbacks;			//!<  # of times CallbackSuspend called with 1 for last parm 
		SIMPLE_ARRAY<DBEDIT_TDATA*> m_TableData;
		OBSERVER *m_Observer;
		RVC::CREDENTIALS *m_Credentials;

		//! Find the table instance for a given form.
		//! 
		//! Returns NULL if none open.
		DBEDIT_TINSTANCE *FindFormInstance (
			const RVC::OBJITEM& ObjItem
			);

		//! Iterate through all instances of a table, calling an instance member function.
		ERRVALUE ForEachTableInstance (
			int (DBEDIT_TINSTANCE::*Func)(void*),
			void *hookdata
			);

		//! Iterate through all instances of a table, calling an instance member function.
		ERRVALUE ForEachTableInstance (
			int (DBEDIT_TINSTANCE::*Func)(void)
			);

		//! Drain the queue.
		//!
		//! Make sure we don't try to drain the queue if we 
	 	//! got called by a callback while draining the queue.
		void DrainQueue ();

		//! Read new records and update open tables.
		void UpdateDialogsInternal ();

		//! Enqueue the database callback. 
		int EnqueueDBEvent (
			const EVENT& event
			);

		//! Dequeue the database callback.
		void DequeueDBEvent ();

		//! It dispatches a Database Callback.
		//!
		//! When the database editor receives a database callback, it can do one of
		//! two things:
		//! (1) If callbacks are not suspended (via DBEDIT::DBCallbackSuspend()), at the
		//! time one is received, it is just dispatched immediately.
		//! (2) If callbacks are suspended it will enqueue the callback, consolidating callbacks
		//! if possible (i.e., if there's already a callback in the queue which would cause it to
		//! update open table windows, it doesn't bother to enqueue another one).  When
		//! callbacks are resumed, it will dispatch all the callbacks in the queue.
		void Dispatch (
			const EVENT& event	// ??? should it be const?
			);

		//!	Allocate the DBEDIT_TDATA for a table.
		//!
		//! Called on initialization and when a new table is added.
		int CreateDBEDIT_TDATA (
			int TableNum
			);

		ERRVALUE OpenDatabaseLow (
			const RVC::OBJITEM& ParentObjItem,
			ELEMTYPE elemtype,
			UINT32 mode,
			const RVC::CREDENTIALS* Credentials
			);

		//! Set the "active" element.
		ERRVALUE SetActiveElementLow (
			ELEMNUM ElemNum, 						//!< Element number or ELEMENTNUM_None
			bool UpdateDialogs,
			INT32 BaseTable = 0					//!< Table to use if this is ELEMTYPE_RECORD
			);

		friend class DBEDIT_TDATA;
		friend class DBEDIT_TINSTANCE;
		friend class DBEDIT_STYLEDIALOG;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(DBEDIT::VIEWFLAGS)
DEFINE_ENUM_OP_INCREMENT(DBEDIT::LABELCOLOR)
#endif

//=====================================================================================================================

//! A class to help insure that we never forget to resume callbacks.
//!
//! Just create an instance of one of these and callbacks will be
//! automatically resumed when it goes out of scope.
class DBEDIT_SUSPENDCALLBACKS {
	public:

		//! Constructor.
		DBEDIT_SUSPENDCALLBACKS (
			DBEDIT *ddata, 
			bool NoQueue = false			//!< Don't even queue callbacks that occur while suspended
			):
			m_ddata(ddata),
			m_NoQueue(NoQueue)
			{
			if (m_ddata != 0) m_ddata->DBCallbackSuspend(m_NoQueue);
			}

		//! Destructor.
		~DBEDIT_SUSPENDCALLBACKS (
			) {
			if (m_ddata != 0) m_ddata->DBCallbackResume(m_NoQueue);
			}

	private:
	   #ifndef GENERATING_DOXYGEN_OUTPUT
		DBEDIT *m_ddata;
		bool m_NoQueue;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
//=====================================================================================================================

#endif
