/**
 * \file mi32/stereorender.h
 * \brief STEREORENDER definitions
 *
 * \if NODOC
 * $Id: stereorender.h_v 1.2 2012/04/09 22:51:31 mju Exp $
 *
 * $Log: stereorender.h_v $
 * Revision 1.2  2012/04/09 22:51:31  mju
 * Add members for no terrain highlight.
 *
 * Revision 1.1  2012/03/29 20:09:37  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_STERERENDER_H
#define	INC_MI32_STERERENDER_H

#ifndef	INC_MI32_STEREOVIEW_H
#include <mi32/stereoview.h>
#endif

#ifndef	INC_MI32_COLOR_H
#include <mi32/color.h>
#endif


//! Stereo rendering parameters.
class STEREORENDER {
	public:

		//! Constructor.
		STEREORENDER ();

		//! Get "amount" of "no terrain" color to use.
		//! @return Amount to use (0-1) relative to image pixel value.
		double GetAmtColorNoTerrain () const
			{ return (m_AmtColorNoTerrain); }

		//! Get color for highlighting "no terrain" areas.
		const COLOR& GetColorNoTerrain () const
			{ return (m_ColorNoTerrain); }

		//! Get maximum relief exaggeration.
		double GetMaxReliefExaggeration () const
			{ return (m_MaxReliefExaggeration); }

		//! Set percentage of depth to render as "beyond" the screen.
		double GetPctDepthBeyondScreen () const
			{ return (m_PctDepthBeyondScreen); }

		//! Get relative depth scale.
		double GetRelativeDepthScale () const
			{ return (m_RelativeDepthScale); }

		//! Get which view to "shift".
		STEREOVIEW GetShiftView () const
			{ return (m_ShiftView); }

		//! Read values from Ini file.
		void IniRead (const char *IniGroup = 0);

		//! Write values to Ini file.
		void IniWrite (const char *IniGroup = 0) const;

		//! Determine if stereo rendering is enabled.
		bool IsEnabled () const
			{ return (m_ShiftView != STEREOVIEW_None); }

		//! Set color and amount to use for "no terrain" areas.
		void SetColorNoTerrain (
			const COLOR& ColorNoTerrain,		//!< Highlighting color
			double amount							//!< Amount (0=none to 1=all)
			) { m_ColorNoTerrain = ColorNoTerrain; m_AmtColorNoTerrain = amount; }

		//! Set maximum relief exaggeration.
		void SetMaxReliefExaggeration (
			double MaxReliefExaggeration
			) { m_MaxReliefExaggeration = MaxReliefExaggeration; }

		//! Set percentage of depth to render as "beyond" the plane of the screen.
		void SetPctDepthBeyondScreen (
			double PctDepthBeyondScreen
			) { m_PctDepthBeyondScreen = PctDepthBeyondScreen; }

		//! Set relative depth scale.
		void SetRelativeDepthScale (
			double RelativeDepthScale
			) { m_RelativeDepthScale = RelativeDepthScale; }

		//! Set view to shift.
		void SetShiftView (
			STEREOVIEW ShiftView
			) { m_ShiftView = ShiftView; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		double m_RelativeDepthScale;
		double m_MaxReliefExaggeration;
		double m_PctDepthBeyondScreen;
		double m_AmtColorNoTerrain;
		COLOR m_ColorNoTerrain;
		STEREOVIEW m_ShiftView;
		#endif
	};

#endif	// INC_MI32_STERERENDER_H
