/**
 * \file mi32/xmenuitemarray.h
 * \brief X MENUITEMARRAY definitions
 *
 * \if NODOC
 * $Id: xmenuitemarray.h_v 1.2 2011/11/08 14:17:59 mju Exp $
 *
 * $Log: xmenuitemarray.h_v $
 * Revision 1.2  2011/11/08 14:17:59  mju
 * Fix dox.
 *
 * Revision 1.1  2011/11/03 13:11:12  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_XMENUITEMARRAY_H
#define	INC_MI32_XMENUITEMARRAY_H

#ifndef INC_MI32_XDEFNS_H
	#include <mi32/xdefns.h>
#endif

#ifndef INC_MI32_SIMPLEAR_H
	#include <mi32/simplear.h>
#endif

//------------------------------------------------------------------------------

//! Class to contain array of MENUITEMs.
class MENUITEMARRAY : public SIMPLE_ARRAY<MENUITEM> {
	public:

		//! Append single MENUITEM.
		ERRVALUE Append (
			const MENUITEM& item			//!< Item to append
			) { Unterminate(); SIMPLE_ARRAY<MENUITEM>::Append(&item,1); Terminate(); return (0); }

		//! Append terminated MENUITEM array to this array.
		ERRVALUE Append (
			const MENUITEM *items		//!< Terminated MENUITEM array
			) { Unterminate(); return (SIMPLE_ARRAY<MENUITEM>::Append(items,MxMenuGetNumItems(items)));}

		//! Determine if array is properly "terminated".
		bool IsTerminated (
			) const { return (GetNumItems() > 0 && GetItem(GetNumItems()-1).label == 0); }

		//! Terminate array if not already.
		ERRVALUE Terminate (
			) { if (!IsTerminated()) return (Resize(GetNumItems()+1,true,true)); else return (0); }

		//! Unterminate array in preparation for adding items to end.
		void Unterminate (
			) { if (IsTerminated()) Resize(GetNumItems()-1); }
	};

//------------------------------------------------------------------------------

#endif	// INC_MI32_XMENUITEMARRAY_H
