/**
 * \file mi32/xydig.h
 * \brief Application include file for X-Y digitizer functions
 *
 * \if NODOC
 * $Id: xydig.h_v 1.26 2012/01/17 22:05:21 mju Exp $
 *
 * $Log: xydig.h_v $
 * Revision 1.26  2012/01/17 22:05:21  mju
 * Remove old mxyd prototypes.
 *
 * Revision 1.25  2011/11/08 14:22:24  mju
 * Remove redundant hdr.
 *
 * Revision 1.12  1998/12/21 14:57:18  mju
 * \endif
**/

#ifndef	INC_MI32_XYDIG_H
#define	INC_MI32_XYDIG_H

#ifndef	INC_MI32_POINT_H
#include <mi32/point.h>
#endif


struct XYDIGTYPE {					/* Accessed through Read/WriteDigRef() */
	INT32 resolution;					//!<  Resolution of digitizer in ppi 
	INT32 spare1;						//!<  Padding so DPOINT2Ds are alligned 
	INT16 leftbtn;						//!<  Value of left button 
	INT16 rightbtn;					//!<  Value of right button 
	INT16 topbtn;						//!<  Value of top button (wintab) 
	INT16 bottombtn;					//!<  Value of bottom button (wintab) 
	INT16 middlebtn;					//!<  Value of middle button / both buttons 
	INT16 calibtomap;					//!<  Flag - set if default map corners exist 
	DPOINT2D lowleft;					//!<  Default lower left corner of map 
	DPOINT2D upright;					//!<  Default upper right corner of map 
	};

#if defined(__cplusplus)
extern "C" {
#endif

//! Initialize wintab.
int MwintabInit ();

//! Free wintab.
int MwintabFree ();

//! Get position & button status.
//! @return button status.
//! return -1 if no coordinate packet was available.
int MwintabGetEvent (
	INT32* x,
	INT32* y,
	int BadEventValue							//!< return (-1) or (0) for "bad events" (when coordinate packet is not available)
	);

//! Get status of digitizer.
//! @return digitizer status
int MwintabGetDigStatus ();

#if defined(__cplusplus)
}
#endif

#endif	//!<  INC_MI32_XYDIG_H 
