/**
 * \file rvc/association.h 
 * \brief interface for the RVC::ASSOCIATION class.
 *
 * \if NODOC
 * $Id: association.h_v 1.3 2011/12/06 22:04:43 mju Exp $
 *
 * $Log: association.h_v $
 * Revision 1.3  2011/12/06 22:04:43  mju
 * Remove redundant hdr.
 *
 * Revision 1.2  2008/02/20 17:57:03  scowan
 * Added settings and getting association type.
 *
 * Revision 1.1  2008/02/11 17:18:04  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef INC_RVC_ASSOCIATION_H
#define INC_RVC_ASSOCIATION_H

#ifndef INC_RVC_SIMPLE_H
	#include <rvc/simple.h>
#endif

namespace RVC {

//! Manages RVC::OBJTYPE_Association RVC objects.
//!
//!	RVC Object Types that this class supports: RVC::OBJTYPE_Association.
//!
//!	Default RVC Object Type for this class: RVC::OBJTYPE_Association.
//!
//!	Valid parents for this class:
//!		- All object types EXCEPT for RVC::OBJTYPE_File, RVC::OBJTYPE_Folder, RVC::OBJTYPE_Container, RVC::OBJTYPE_Association
//!
//!	Read(), Write() and Make() methods inherited from RVC::SIMPLE class.  View RVC::SIMPLE docs for inherited methods
class ASSOCIATION : public RVC::SIMPLE {
	public:
		enum TYPES {
			TYPE_Unknown = 0,
			TYPE_KML = 1,
			TYPE_URL = 2,
			TYPE_LAST
			};

		//! Default constructor.
		ASSOCIATION (
			);

		//! Copy constructor.
		ASSOCIATION (
			const RVC::ASSOCIATION& rhs
			);

		//! Destructor.
		virtual ~ASSOCIATION (
			);

		//! Assignment.
		RVC::ASSOCIATION& operator= (
			const RVC::ASSOCIATION& rhs
			);

		//! Get the current association parms
		//! @return MISTRING of Association parms
		const MISTRING& GetAssociationParms (
			) const { return (m_AssociationParm); }

		//! Set the association type
		TYPES GetAssociationType (
			) const { return (m_AssociationType); }

		//! Set the association parms
		void SetAssociationParms (
			const MISTRING& AssociationParm
			) { m_AssociationParm = AssociationParm; }
			
		//! Set the association type
		void SetAssociationType (
			TYPES AssociationType
			) { m_AssociationType = AssociationType; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		// Implementation of RVC::OBJECT virtual methods
		virtual OBJTYPE v_GetDftObjectType () const;

		// Implementation of RVC::SIMPLE virtual methods
		virtual ERRVALUE v_GetDftNameDesc (const OBJECT& parent, DESCRIPTOR& Descriptor);
		virtual ERRVALUE v_MakeData (RVC::OBJMAKEPARMS& ObjMakeParms);
		virtual ERRVALUE v_ReadData (RVC::OBJOPENPARMS& ObjOpenParms);
		virtual ERRVALUE v_WriteData (RVC::OBJOPENPARMS& ObjOpenParms);

		MISTRING m_AssociationParm;
		TYPES m_AssociationType;
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

}		// End of RVC namespace

#endif 	// INC_RVC_ASSOCIATION_H
