/**
 * \file rvc/folder.h 
 * \brief interface for the RVC::FOLDER class.
 *
 * \if NODOC
 * $Id: folder.h_v 1.14 2007/10/11 19:37:39 scowan Exp $
 *
 * $Log: folder.h_v $
 * Revision 1.14  2007/10/11 19:37:39  scowan
 * Fixed sml manditory credential issue.
 *
 * Revision 1.13  2006/06/28 15:33:52  scowan
 * Changed make to take a non-const objitem.
 *
 * Revision 1.12  2005/12/19 17:02:23  linux32build!dwilliss
 * Credentials can now be made mandatory by setting a #define
 *
 * Revision 1.11  2005/12/09 17:29:27  scowan
 * Updated to use credentials.
 *
 * Revision 1.10  2005/06/17 17:50:13  scowan
 * Removed deprecated make method.
 *
 * Revision 1.9  2005/01/19 21:38:06  scowan
 * Added support for descriptor.
 *
 * Revision 1.8  2003/10/07 17:32:51  scowan
 * Object virtual method changes.
 *
 * Revision 1.7  2003/09/30 16:52:06  dwilliss
 * Doxygen
 *
 * Revision 1.6  2003/09/25 21:20:04  dwilliss
 * Fixed doxygen comment
 *
 * Revision 1.5  2003/09/15 13:49:45  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.4  2001/06/14 17:46:53  scowan
 * Added to RVC namespace.
 *
 * Revision 1.3  2001/06/13 14:42:59  scowan
 * objtype enum change.
 * Added new make method.
 *
 * Revision 1.2  2000/10/19 17:52:36  scowan
 * Checked in for vacation.
 *
 * Revision 1.1  2000/09/29 14:04:56  scowan
 * Initial revision
 *
 * \endif
**/

//!
//!	\class RVC::FOLDER rvc/folder.h
//!	\brief Manages RVC::OBJTYPE_Folder RVC objects.  
//!
//!	The interface currently wraps the MfMakeFolder function.
//!	
//!	This class inherits from RVC::OBJECT
//!	
//!	RVC Object Types that this class supports: RVC::OBJTYPE_Folder.
//!
//!	Default RVC Object Type for this class: RVC::OBJTYPE_Folder.
//!
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_File
//!		- RVC::OBJTYPE_Folder

#ifndef INC_RVC_FOLDER_H
#define INC_RVC_FOLDER_H

#ifndef INC_RVC_OBJECT_H
	#include <rvc/object.h>
#endif

namespace RVC {

class FOLDER : public RVC::OBJECT {  
	public:

		//! Default constructor
		FOLDER (
			);

		//! Copy constructor
		FOLDER (
			const RVC::FOLDER& obj
			);

		//! Destructor
		virtual ~FOLDER (
			);

		//! Assignment
		RVC::FOLDER& operator= (
			const RVC::FOLDER& rhs
			);

		//! Make a child rvc folder object from a given parent with a name and description
		ERRVALUE Make (
			const RVC::OBJECT& parent,
			const RVC::DESCRIPTOR& Descriptor,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Make a child rvc folder object from a given RVCOBJITEM
		ERRVALUE Make (
			RVC::OBJITEM& ObjItem,								//!< Parent object and DESCRIPTOR of object to create, new location RETURNED
#ifdef RVC_CREDENTIALS_MANDATORY
			const char* Source,
#else
			const char* Source = 0,
#endif
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		//! Implementation of RVC::OBJECT virtual methods
		virtual OBJTYPE v_GetDftObjectType () const;
		
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}		//! End of RVC namespace

#endif		//!< INC_RVC_FOLDER_H
