/**
 * \file rvc/gettablefieldfilter.h
 * \brief Definitions for RVC::GETTABLEFIELDFILTER class
 *
 * \if NODOC
 * $Id: gettablefieldfilter.h_v 1.1 2006/10/13 17:50:49 mju Exp $
 *
 * $Log: gettablefieldfilter.h_v $
 * Revision 1.1  2006/10/13 17:50:49  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_GETTABLEFIELDFILTER_H
#define	INC_RVC_GETTABLEFIELDFILTER_H

#ifndef	INC_RVC_DBTABLE_H
#include <rvc/dbtable.h>
#endif

namespace RVC {

//! Base class for table/field selection filter.
class GETTABLEFIELDFILTER {
	public:

		//! Default constructor.
		GETTABLEFIELDFILTER (
			);

		//! Clear current TEXTID and error value settings.
		void Clear (
			) { m_TextID = TEXTID__None; m_ErrValue = 0; }

		//! Get current error value.
		ERRVALUE GetErrValue (
			) const { return (m_ErrValue); }
			
		//! Get message based on the stored TEXTID and error value.
		//! If both and error value and a TEXTID are set the error value will take precedence.
		MISTRING GetMessage (
			TEXTID TextID = TEXTID__None		//!< Text ID to use for message if none previously set
			) const;
			
		//! Get current TEXTID setting.
		TEXTID GetTextID (
			) const { return (m_TextID); }
			
		//! Determine if the table is selectable.
		//! @return 'true' if selectable, 'false' if not.
		//! Default implementation always returns 'true'.
		virtual bool IsTableValid (
			const RVC::DBTABLE& TableObj
			);

		//! Determine if the field is selectable.
		//! @return 'true' if selectable, 'false' if not.
		//! Default implementation always returns 'true'.
		virtual bool IsFieldValid (
			const RVC::DBTABLE& TableObj,
			INT32 FieldNum,
			const RVC::DBTABLE::FIELDINFO& FieldInfo
			);

		//! Set error value.
		void SetErrValue (
			ERRVALUE err
			) { m_ErrValue = err; }
			
		//! Set the current TEXTID for this filter
		void SetTextID (
			TEXTID TextID
			) { m_TextID = TextID; }
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		TEXTID m_TextID;
		ERRVALUE m_ErrValue;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}	// End namespace RVC

#endif	// INC_RVC_GETTABLEFIELDFILTER_H
