/**
 * \file rvc/oprastertogeomline.h
 * \brief Operation to convert raster to geometric by lineary tracing
 *
 * \if NODOC
 * $Id: oprastertogeomline.h_v 1.2 2011/09/02 12:53:13 mju Exp $
 *
 * $Log: oprastertogeomline.h_v $
 * Revision 1.2  2011/09/02 12:53:13  mju
 * Dox hdr.
 *
 * Revision 1.1  2006/08/24 17:21:05  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_OPRASTERTOGEOMLINE_H
#define	INC_RVC_OPRASTERTOGEOMLINE_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

namespace RVC {

class OBJITEM;

namespace OP {

//=====================================================================================================================

//! Converter for performing raster to geometric conversion by lineary tracing.
class RASTER_TO_GEOMETRIC_LINE {
	public:
		enum FLAGS {
			FLAG_Trace1Cells = 0x0000,
			FLAG_Trace0Cells = 0x0001
			};

		//! Convert source to target using current settings.
		//! The target object type will be determined from the specified target location settings.
		static ERRVALUE Process (
			const RVC::OBJITEM& ObjItemSrc,				//!< Source location
			RVC::OBJITEM& ObjItemTgt,						//!< Target location passed/returned
			double ThinFactor,								//!< Line thinning factor
			FLAGS flags = FLAG_Trace1Cells
			);

	};

//=====================================================================================================================

}	// End namespace OP
}	// End namespace RVC

#endif	// INC_RVC_OPRASTERTOGEOMLINE_H
