/**
*	\file	rvc/shapetoolkit.h
*
*	\if NODOC
*	$Id: shapetoolkit.h_v 1.3 2011/12/23 14:01:31 mju Exp $
*
*	$Log: shapetoolkit.h_v $
*	Revision 1.3  2011/12/23 14:01:31  mju
*	Remove redundant hdrs.
*
*	Revision 1.2  2008/05/29 16:24:38  scowan
*	Removed unused callbacks.
*
*	Revision 1.1  2008/05/23 22:50:41  scowan
*	Initial revision
*	\endif
**/

#ifndef	INC_RVC_SHAPETOOLKIT_H
#define	INC_RVC_SHAPETOOLKIT_H

#ifndef	INC_RVC_SHAPE_H
	#include <rvc/shape.h>
#endif

#ifndef	INC_RVC_STYLE_H
	#include <rvc/style.h>
#endif

#ifndef	INC_RVC_DBSHAPE_H
	#include <rvc/dbshape.h>
#endif

namespace RVC {

#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward Declarations
class SHAPETOOLKITOBSERVER;
#endif // GENERATING_DOXYGEN_OUTPUT

class SHAPETOOLKIT : public SUBJECT<SHAPETOOLKITOBSERVER> {
	public:

		enum TOOLFLAG {
			TOOLFLAG_None = 0x000,					//!< No flag state
			TOOLFLAG_NoQtree = 0x001,				//!< Do not maintain qtrees
			TOOLFLAG_NoDatabase = 0x008,			//!< Do not maintain databases
			TOOLFLAG_NoCreateTables = 0x020,		//!< Do not create databases
			TOOLFLAG_NoSaveQTree = 0x100			//!< Do not save search tree in Close() or Stop()
			};

		SHAPETOOLKIT ();

		virtual ~SHAPETOOLKIT ();

		INT32 AddElement (
			SHAPE::ELEMENT& Element
			);

		ERRVALUE ChangeElement (
			INT32 ElemNum, 
			SHAPE::ELEMENT& Element
			);

		ERRVALUE Close (
			);

		ERRVALUE CopyElement (
			INT32 ElemNum,
			const TRANS2D_AFFINE& OldToNew
			);

		ERRVALUE CopyElements (
			const BITSET& set,
			const TRANS2D_AFFINE& OldToNew
			);

		ERRVALUE DeleteElement (
			INT32 ElemNum
			);
			
		ERRVALUE ExplodeElement (
			INT32 ElemNum
			);

		double GetZValue (
			INT32 ElemNum
			);

		ERRVALUE MoveElement (
			INT32 ElemNum,
			const TRANS2D_AFFINE& OldToNew
			);

		ERRVALUE MoveElements (
			const BITSET& set,
			const TRANS2D_AFFINE& OldToNew
			);

		ERRVALUE Open (
			const OBJITEM& ObjItem,
			TOOLFLAG flags
			);

		ERRVALUE Open (
			const SHAPE& ShapeObj,
			TOOLFLAG flags
			);

		ERRVALUE RecoverFromUndo (
			);

		ERRVALUE ReverseLinePoints (
			INT32 ElemNum
			);

		ERRVALUE Save (
			OBJITEM& ObjItem
			);

		TOOLFLAG SetFlags (
			TOOLFLAG NewFlags
			);

		void SetSearchDist (
			double dist
			) { m_SearchDist = dist; }

		ERRVALUE SetZValue (
			INT32 ElemNum, 
			double zvalue
			);

		ERRVALUE StraightenLine (
			INT32 ElemNum
			);

		ERRVALUE Validate (
			);

	private:
	
		#ifndef GENERATING_DOXYGEN_OUTPUT
		SHAPETOOLKIT (const SHAPETOOLKIT& rhs);
		SHAPETOOLKIT operator= (const SHAPETOOLKIT& rhs);
		
		class LOCALOBSERVER : public DBASE::OBSERVER {
			public:
			
				LOCALOBSERVER (
					DBASE_SHAPE& Database,
					INT32& ElemIDTableNum
					) : DBASE::OBSERVER(Database), m_ElemIDTableNum(ElemIDTableNum) {}
			
			private:
				virtual void OnTableDropEnd (INT32 TableNum);

				INT32& m_ElemIDTableNum;
			};
			
		SHAPE m_ShapeObj;
		INT32 m_ElemIDTableNum;			//!< Element ID table number
		TOOLFLAG m_ToolkitFlags;
		DBASE_SHAPE m_DBaseObj;
		DBTABLE::RECORD m_Record;
		SIMPLE_ARRAY<DBLISTELMT> m_DBList;
		double m_CurZValue;						//!<  Current 3DXY Z value to assign to the element 
		double m_SearchDist;
	   bool m_HasDftZValue;						//!<  Add default Z Value to element that is added 
		bool m_HasMinMax;
		bool m_IgnoreAddDBList;
		bool m_HasSearchTree;
		LOCALOBSERVER* m_Observer;
		#endif // GENERATING_DOXYGEN_OUTPUT

		void CallOnAdd (INT32 ElemNum);
		void CallOnDelete (INT32 ElemNum);
		void CallOnMoveBegin (INT32 ElemNum);
		void CallOnMoveEnd (INT32 ElemNum);
		
		ERRVALUE AddElemDBList (INT32 ElemNum);
		ERRVALUE AddQTreeElem (INT32 elemnum, const DRECT2D& Extents);
		ERRVALUE BuildQTree(INT32 BlockNum);

		ERRVALUE ChangeQTreeElem (
			INT32 ElemNum,
			const DRECT2D& Extents
			) { return (CopyQTreeElem(ElemNum, ElemNum, Extents)); }

		INT32 CopyDBRecordRaw (INT32 TableNum, INT32 SourceRecord, INT32 DestRecord);
		ERRVALUE CopyDBRecords (INT32 SourceElem, INT32 DestElem);
		ERRVALUE CopyQTreeElem (INT32 source, INT32 dest, const DRECT2D& Extents);
		ERRVALUE DeleteQTreeElem (INT32 elemnum, const DRECT2D& Extents);
		
	};
	
DEFINE_ENUM_OP_BITWISE(SHAPETOOLKIT::TOOLFLAG)

class SHAPETOOLKITOBSERVER : public OBSERVER_BASE {
	public:

		SHAPETOOLKITOBSERVER (
			SHAPETOOLKIT& ctk
			) :
			OBSERVER_BASE(ctk)
			{}
			
		virtual ~SHAPETOOLKITOBSERVER (
			) {}
			
		virtual void OnAdd (
			INT32 ElemNum
			);
			
		virtual void OnDelete (
			INT32 ElemNum
			);
			
		virtual void OnMoveBegin (
			INT32 ElemNum
			);

		virtual void OnMoveEnd (
			INT32 ElemNum
			);

	};

}		// End of RVC namespace

#endif		//	INC_RVC_SHAPETOOLKIT_H
